local ezInterrupt = LibStub("AceAddon-3.0"):NewAddon("ezInterrupt", "AceConsole-3.0", "AceEvent-3.0")
local LSM = LibStub("LibSharedMedia-3.0")
local outputChannel = "NONE"
local lastSoundAlert = 0
local msgTags = {
	["[spell]"]		= nil,
	["[target]"]	= nil,
	["[interrupt]"] = nil,
}

LSM:Register("sound", "Shay's Bell", [[Sound\Spells\ShaysBell.ogg]])
LSM:Register("sound", "Simon Chime", [[Sound\Doodad\SimonGame_LargeBlueTree.wav]])
LSM:Register("sound", "Humm", [[Sound\Spells\SimonGame_Visual_GameStart.wav]])
LSM:Register("sound", "Short Circuit", [[Sound\Spells\SimonGame_Visual_BadPress.wav]])
LSM:Register("sound", "ezInterrupt: Beep", [[Interface\Addons\ezInterrupt\media\beep.ogg]])
LSM:Register("sound", "ezInterrupt: Ding", [[Interface\Addons\ezInterrupt\media\ding.ogg]])

local defaults = {
    profile = {
        enableAddon = true,
		enableAnnouncing = true,
		enableWarning = true,
		enableWarningSound = true,
		enableWarningIcon = true,
		enableWhispering = true,
		enableCustomChannel = true,
		tracking = "target",
		whisperTarget = nil,
		customChannel = nil,
		customMessage = "Interrupted [spell]",
		slashSay = true,
		slashParty = true,
		slashRaid = false,
		useInRaid = true,
		useInParty = true,
		useInBattleground = false,
		useInArena = false,
		useOutsideInstance = true,
		soundFile = "ezInterrupt: Ding",
		filterMode = 1, -- 1 means don't filter, 2 means filter through blackList, 3 means filter through whitelist 
		whiteList = {},
		blackList = {},
		warningIconPosition = {"CENTER",nil,"CENTER",0,0},
		warningIconSize = 44,
		warningIconBlend = "DISABLE",
    },
}

local options = {
    name = "ezInterrupt Configuration",
    handler = ezInterrupt,
    type = "group",
    args = {
		enable = {
			type = "toggle",
			name = "Enable Addon",
			width = "full",
			set = function(info, val) ezInterrupt.db.profile.enableAddon = val if ezInterrupt.db.profile.enableAddon then ezInterrupt:Enable() else ezInterrupt:Disable() end end,
			get = function(info) return ezInterrupt.db.profile.enableAddon end,
			order = 1
		},
		announcing = {
			type = "toggle",
			name = "Announce interrupts",
			width = "full",
			set = function(info, val) ezInterrupt.db.profile.enableAnnouncing = val ezInterrupt:UpdateCombatLogParsing() end,
			get = function(info) return ezInterrupt.db.profile.enableAnnouncing end,
			order = 2
		},
		warning = {
			type = "toggle",
			name = "Spellcast warnings",
			width = "full",
			set = function(info, val) ezInterrupt.db.profile.enableWarning = val ezInterrupt:UpdateCombatLogParsing() end,
			get = function(info) return ezInterrupt.db.profile.enableWarning end,
			order = 3
		},
		output = {
			type = "group",
			name = "Announcement Options",
			order = 5,
			inline = true,
			args = {
				header1 = {
					type = "description",
					name = "Check the output channels that you want to make available to the addon. It will select the appropriate channel automatically and only report once per interrupt. Raid takes priority over Party which takes priority over Say.",
					order = 1
				},
				slashsay = {
					type = "toggle",
					name = "Say",
					width = "half",
					set = function(info, val) ezInterrupt.db.profile.slashSay = val ezInterrupt:UpdateOutputChannel() end,
					get = function(info) return ezInterrupt.db.profile.slashSay end,
					order = 4
				},
				slashparty = {
					type = "toggle",
					name = "Party",
					width = "half",
					set = function(info, val) ezInterrupt.db.profile.slashParty = val ezInterrupt:UpdateOutputChannel() end,
					get = function(info) return ezInterrupt.db.profile.slashParty end,
					order = 3
				},
				slashraid = {
					type = "toggle",
					name = "Raid",
					width = "half",
					set = function(info, val) ezInterrupt.db.profile.slashRaid = val ezInterrupt:UpdateOutputChannel() end,
					get = function(info) return ezInterrupt.db.profile.slashRaid end,
					order = 2
				},
				header2 = {
					type = "description",
					name = "In addition to the above, the addon can also whisper another player when you interrupt or show your interrupts in a custom channel.",
					order = 5
				},
				whispering = {
					type = "toggle",
					name = "Enable Whispering",
					set = function(info, val) ezInterrupt.db.profile.enableWhispering = val end,
					get = function(info) return ezInterrupt.db.profile.enableWhispering end,
					order = 6
				},
				whispertarget = {
					type = "input",
					name = "Which Player?",
					desc = "Input the name of the player you want to whisper to.",
					set = function(info, val) if val == "" then ezInterrupt.db.profile.whisperTarget = nil else ezInterrupt.db.profile.whisperTarget = val end end,
					get = function(info) return ezInterrupt.db.profile.whisperTarget end,
					order = 7
				},
				customchannel = {
					type = "toggle",
					name = "Enable Custom Channel",
					set = function(info, val) ezInterrupt.db.profile.enableCustomChannel = val end,
					get = function(info) return ezInterrupt.db.profile.enableCustomChannel end,
					order = 8
				},
				customchannelfield = {
					type = "input",
					name = "Which Channel?",
					desc = "Insert the number of the custom channel you want to report to. Example: 5",
					set = function(info, val) if val == "" then ezInterrupt.db.profile.customChannel = nil else ezInterrupt.db.profile.customChannel = val end end,
					get = function(info) return ezInterrupt.db.profile.customChannel end,
					order = 9
				},
				header3 = {
					type = "description",
					name = "You can construct a customized interrupt message using the following tags:|n|cFFff9000[target]|r will be replaced by the name of the unit that was interrupted.|n|cFFff9000[spell]|r will be replaced with the interrupted spell.|n|cFFff9000[interrupt]|r will be replaced by the name of your interrupt ability.",
					order = 10
				},
				customMessage = {
					type = "input",
					width = "full",
					name = "Message",
					desc = "Example: Interrupted [target]'s [spell] with [interrupt] >>> Player next! <<<",
					set = function(info, val) if val == "" then ezInterrupt.db.profile.customMessage = "Interrupted [spell]" else ezInterrupt.db.profile.customMessage = val end end,
					get = function(info) return ezInterrupt.db.profile.customMessage end,
					order = 12
				},
			}
		},
		usewhere = {
			type = "group",
			name = "Where should ezInterrupt become active?",
			order = 6,
			inline = true,
			args = {
				useinparty = {
					type = "toggle",
					name = "In party instances",
					width = "full",
					set = function(info, val) ezInterrupt.db.profile.useInParty = val ezInterrupt:UpdateCombatLogParsing() end,
					get = function(info) return ezInterrupt.db.profile.useInParty end,
					order = 3,
				},
				useinraid = {
					type = "toggle",
					name = "In raid instances",
					width = "full",					
					set = function(info, val) ezInterrupt.db.profile.useInRaid = val ezInterrupt:UpdateCombatLogParsing() end,
					get = function(info) return ezInterrupt.db.profile.useInRaid end,
					order = 2
				},
				useOutsideInstance = {
					type = "toggle",
					name = "Outside of instances",
					width = "full",
					set = function(info, val) ezInterrupt.db.profile.useOutsideInstance = val ezInterrupt:UpdateCombatLogParsing() end,
					get = function(info) return ezInterrupt.db.profile.useOutsideInstance end,
					order = 1
				},		
				useinbg = {
					type = "toggle",
					name = "In battlegrounds",
					width = "full",
					set = function(info, val) ezInterrupt.db.profile.useInBattleground = val ezInterrupt:UpdateCombatLogParsing() end,
					get = function(info) return ezInterrupt.db.profile.useInBattleground end,
					order = 4
				},		
				useinarena = {
					type = "toggle",
					name = "In arenas",
					width = "full",
					set = function(info, val) ezInterrupt.db.profile.useInArena = val ezInterrupt:UpdateCombatLogParsing() end,
					get = function(info) return ezInterrupt.db.profile.useInArena end,
					order = 5
				},					
			},
		},
		warningoptions = {
			type = "group",
			name = "Warning Options",
			order = 7,
			inline = true,
			args = {
				enablesound = {
					type = "toggle",
					name = "Play Sound",
					width = "full",
					set = function(info, val) ezInterrupt.db.profile.enableWarningSound = val end,
					get = function(info) return ezInterrupt.db.profile.enableWarningSound end,
					order = 1,
				},
				enablevisual = {
					type = "toggle",
					name = "Show Icon",
					width = "full",					
					set = function(info, val) ezInterrupt.db.profile.enableWarningIcon = val end,
					get = function(info) return ezInterrupt.db.profile.enableWarningIcon end,
					order = 2
				},
				tracking = {
					type = "select",
					name = "Track which unit?",
					desc = "Select the unit that you want see warnings for.",
					style = "dropdown",
					width = "full",			
					values = {["target"]="Your target",["focus"]="Your focus target"},
					set = function(info, val) ezInterrupt.db.profile.tracking = val end,
					get = function(info) return ezInterrupt.db.profile.tracking end,
					order = 3
				},
				header4 = {
					type = "description",
					name = "Tip: warning sounds are played through the master volume channel.",
					order = 5,
				},
				selectsound = {
					type = "select", dialogControl = 'LSM30_Sound',
					name = "Which sound file?",
					style = "dropdown",
					width = "full",
					values = AceGUIWidgetLSMlists.sound,
					set = function(info, val) ezInterrupt.db.profile.soundFile = val end,
					get = function(info) return ezInterrupt.db.profile.soundFile end,
					order = 4
				},	
				filtering = {
					type = "select",
					name = "Spell filtering",
					style = "dropdown",
					width = "full",
					values = {[1]="Disabled",[2]="Blacklist Filtering",[3]="Whitelist Filtering"},
					set = function(info, val) ezInterrupt.db.profile.filterMode = val end,
					get = function(info) return ezInterrupt.db.profile.filterMode end,
					order = 6
				},	
				blacklist = {
					type = "input",
					name = "Blacklist: add/remove",
					desc = "Insert a spell name to add it to the blacklist. Add it again to remove it.",
					get = function(info) return nil end,
					set = function(info, val) ezInterrupt:UpdateFilterList(ezInterrupt.db.profile.blackList, "Blacklist", val) end,
					order = 7
				},	
				whitelist = {
					type = "input",
					name = "Whitelist: add/remove",
					desc = "Insert a spell name to add it to the whitelist. Add it again to remove it.",
					get = function(info) return nil end,
					set = function(info, val) ezInterrupt:UpdateFilterList(ezInterrupt.db.profile.whiteList, "Whitelist", val) end,
					order = 9
				},	
				displayblacklist = {
					type = "execute",
					name = "Display Blacklist",					
					func = function(info) ezInterrupt:showFilterListContents(ezInterrupt.db.profile.blackList, "Blacklist") end,
					order = 8
				},		
				displaywhitelist = {
					type = "execute",
					name = "Display Whitelist",					
					func = function(info) ezInterrupt:showFilterListContents(ezInterrupt.db.profile.whiteList, "Whitelist") end,
					order = 10
				},		
				iconlock = {
					type = "execute",
					name = "Toggle icon frame lock",
					desc = "Allows you to drag the warning icon frame with the mouse.",
					width = "full",
					func = function() if not WarningFrame:IsMovable() then WarningFrame:Show() WarningFrame:EnableMouse(true) WarningFrame:SetMovable(true) WarningFrame:RegisterForDrag("LeftButton") else WarningFrame:Hide() WarningFrame:EnableMouse(false) WarningFrame:SetMovable(false) WarningFrame:RegisterForDrag() end end,
					order = 11
				},				
				iconscale = {
					type = "range",
					name = "Warning Icon Scale",
					min = 16,
					max = 256,
					width = "full",
					set = function(info, val) ezInterrupt.db.profile.warningIconSize = val WarningFrame:SetWidth(val) WarningFrame:SetHeight(val) WarningFrame.Icon:SetPoint("TOPLEFT", val/16, -val/16) WarningFrame.Icon:SetPoint("BOTTOMRIGHT", -val/16, val/16)	 end,
					get = function(info) return ezInterrupt.db.profile.warningIconSize end,
					order = 12
				},		
				iconalpha = {
					type = "range",
					name = "Warning Icon Alpha",
					min = 0.1,
					max = 1,
					width = "full",
					isPercent = true,
					set = function(info, val) ezInterrupt.db.profile.warningIconAlpha = val WarningFrame:SetAlpha(val) end,
					get = function(info) return ezInterrupt.db.profile.warningIconAlpha end,
					order = 13
				},	
				blendmode = {
					type = "select",
					name = "Warning Icon Blend Mode",
					style = "dropdown",
					width = "full",
					desc = "Lets you make the warning icon look nicer.",
					values = {["BLEND"]="Standard",["ADD"]="Additive"},
					set = function(info, val) ezInterrupt.db.profile.warningIconBlend = val WarningFrame.Border:SetBlendMode(val) WarningFrame.Icon:SetBlendMode(val) end,
					get = function(info) return ezInterrupt.db.profile.warningIconBlend end,
					order = 14
				},	
			}
		}
	}
}

function ezInterrupt:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("ezInterruptDB", defaults, "Default")
	LibStub("AceConfig-3.0"):RegisterOptionsTable("ezInterrupt", options)
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("ezInterrupt", "ezInterrupt")
	self:SetEnabledState(self.db.profile.enableAddon)
	WarningFrame = CreateFrame("FRAME", "ezInterruptMainFrame", UIParent)
	WarningFrame:SetFrameStrata("HIGH")
	WarningFrame:SetPoint(unpack(self.db.profile.warningIconPosition))
	WarningFrame:SetWidth(self.db.profile.warningIconSize)
	WarningFrame:SetHeight(self.db.profile.warningIconSize)
	WarningFrame:SetClampedToScreen(true)
	WarningFrame:SetAlpha(self.db.profile.warningIconAlpha)
	WarningFrame:Hide()
	WarningFrame.Icon = WarningFrame:CreateTexture("ezInterruptIcon", "background")
	WarningFrame.Icon:SetDrawLayer("background", 0)
	WarningFrame.Icon:SetPoint("TOPLEFT", self.db.profile.warningIconSize/16, -self.db.profile.warningIconSize/16)
	WarningFrame.Icon:SetPoint("BOTTOMRIGHT", -self.db.profile.warningIconSize/16, self.db.profile.warningIconSize/16)	
	WarningFrame.Icon:SetTexCoord(0.0625,0.9375,0.0625,0.9375)
	WarningFrame.Icon:SetTexture("Interface\\Icons\\Spell_Fire_SealOfFire")	
	WarningFrame.Icon:SetBlendMode(self.db.profile.warningIconBlend)
	WarningFrame.Border = WarningFrame:CreateTexture("ezInterruptIconBorder", "border")
	WarningFrame.Border:SetAllPoints()
	WarningFrame.Border:SetTexture("Interface\\Addons\\ezInterrupt\\media\\edge.tga")
	WarningFrame.Border:SetVertexColor(1, 0, 0)
	WarningFrame.Border:SetBlendMode(self.db.profile.warningIconBlend)
	WarningFrame:SetScript("OnDragStart", function(self, button) self:StartMoving() end)
	WarningFrame:SetScript("OnDragStop", function(self, button) self:StopMovingOrSizing() ezInterrupt.db.profile.warningIconPosition = {self:GetPoint(1)} end)
	WarningFrame:SetScript("OnHide", function(self) self:StopMovingOrSizing() ezInterrupt.db.profile.warningIconPosition = {self:GetPoint(1)} end)
	local class = UnitClass("player")	
	if class == "Warrior" then
		self.InterruptList = {
			[6552] = true, -- Pummel
			[72] = true, -- Shield Bash
		}	
	elseif class == "Druid" then
		self.InterruptList = {
			[80964] = true, -- Skull Bash (Bear)
			[80965] = true, -- Skull Bash (Cat)
		}
	elseif class == "Rogue" then
		self.InterruptList = {
			[1766] = true, -- Kick
		}
	elseif class == "Paladin" then
		self.InterruptList = {
			[96231] = true, -- Rebuke
		}
	elseif class == "Deathknight" then
		self.InterruptList = {
			[47528] = true, -- Mind Freeze
		}
	elseif class == "Shaman" then
		self.InterruptList = {
			[57994] = true, -- Wind Shear
		}
	elseif class == "Mage" then
		self.InterruptList = {
			[2139] = true, -- Counterspell
		}
	elseif class == "Warlock" then
		self.InterruptList = {
			[19647] = true, -- Spell Lock, bugged and shows as usable even if the hunter doesn't have a corresponding pet
		}
	elseif class == "Hunter" then
		self.InterruptList = {
			[34490] = true, -- Silencing Shot, bugged as it shows as usable even if the hunter doesn't have the talent
			-- [50479] = true, -- Nether Shock (Nether Ray pets), bugged and shows as usable even if the hunter doesn't have a corresponding pet
			-- [26090] = true, -- Pummel (Gorilla pets), bugged and shows as usable even if the hunter doesn't have a corresponding pet
		}
	elseif class == "Priest" then
		self.InterruptList = {
			[15487] = true, -- Silence, bugged as it shows as usable even if the priest doesn't have the talent
		}
	end	
end

function ezInterrupt:OnEnable()
	self:RegisterEvent("PLAYER_ENTERING_WORLD")
	self:RegisterEvent("PARTY_MEMBERS_CHANGED", "UpdateOutputChannel")
	self:UpdateOutputChannel() 
	self:UpdateCombatLogParsing()
end

function ezInterrupt:OnDisable()
	self:UnregisterAllEvents()		
end

function ezInterrupt:showFilterListContents(list, listname)
	if #list > 0 then
		DEFAULT_CHAT_FRAME:AddMessage("|cFFff9000ez|r|cFFccff1dInterrupt:|r the "..listname.." contains")
		for i,v in ipairs(list) do 
		DEFAULT_CHAT_FRAME:AddMessage(i..". |cFF5dd9f8"..v.."|r")
		end
	else
		DEFAULT_CHAT_FRAME:AddMessage("|cFFff9000ez|r|cFFccff1dInterrupt:|r the "..listname.." is empty")
	end
end

function ezInterrupt:CheckForCast(throttle)
	if UnitIsEnemy("player", self.db.profile.tracking) then --
		local spellName = UnitCastingInfo(self.db.profile.tracking) or UnitChannelInfo(self.db.profile.tracking)
		local immune = select(9, UnitCastingInfo(self.db.profile.tracking)) or select(8, UnitChannelInfo(self.db.profile.tracking))
		if spellName and not immune and self:IsSpellAllowed(spellName) then -- don't check IsInterruptReady() here 
			self:StartAlert(throttle)
		end
	end
end

-- adding some throttling is probably good
local function OnUpdate(self,elapsed)
	self.total = self.total + elapsed
	local cd = GetSpellCooldown(self.spell)
	if cd == 0 then -- handle anything that reduces or resets the cooldown on the interrupt and also account for casts starting between the first early CheckForCast and the interrupt actually coming off cooldown
		ezInterrupt.InterruptList[self.spell] = true
		ezInterrupt:CheckForCast(true)
		self:SetScript("OnUpdate",nil)
	elseif self.check == true and self.total >= self.limit then
		self.check = false
		ezInterrupt:CheckForCast(true)
	end
end

local frame = CreateFrame("Frame")
frame:SetScript("OnEvent", function(self,event)
	for k,v in pairs(ezInterrupt.InterruptList) do
		local start, cd, enable = GetSpellCooldown(k)
		if cd > 0 and ezInterrupt.InterruptList[k] == true then
			ezInterrupt.InterruptList[k] = false
			local f = CreateFrame("Frame")
			f.total = 0
			f.limit = cd-0.1 -- get an early warning
			f.spell = k
			f.check = true
			f:SetScript("OnUpdate", OnUpdate)
		end
	end
end)

function ezInterrupt:IsInterruptReady()
	for k,v in pairs(self.InterruptList) do
		local ready, noresource = IsUsableSpell(k) -- Required for druid forms, defstance+shield for shield bash, etc.
		if (ready or noresource) and GetSpellCooldown(k) == 0 then
			return true
		end
	end
	return false
end

function ezInterrupt:IsSpellAllowed(spellName)
	if (self.db.profile.filterMode == 2 and tContains(self.db.profile.blackList, spellName)) or (self.db.profile.filterMode == 3 and not tContains(self.db.profile.whiteList, spellName)) then
		return false
	else
		return true
	end
end

function ezInterrupt:StartAlert(throttle)
	if self.db.profile.enableWarningSound then 
		if throttle then -- this prevents two sound alerts in 100 ms from the OnUpdate script
			if lastSoundAlert+0.25 > GetTime() then return end
			lastSoundAlert = GetTime()
		end
		PlaySoundFile(LSM:Fetch("sound", self.db.profile.soundFile), "MASTER") end
	if self.db.profile.enableWarningIcon then
		local icon = select(4, UnitCastingInfo(ezInterrupt.db.profile.tracking)) or select(4, UnitChannelInfo(ezInterrupt.db.profile.tracking))
		WarningFrame.Icon:SetTexture(icon)
		WarningFrame:Show()		
	end
end

function ezInterrupt:StopAlert()
	WarningFrame:Hide()
end

-- UNIT_SPELLCAST_STOP also handles UNIT_SPELLCAST_CHANNEL_STOP
function ezInterrupt:UNIT_SPELLCAST_STOP(event, unitID, spellName)
	if unitID == ezInterrupt.db.profile.tracking and UnitIsEnemy("player", unitID) then
		self:StopAlert()
	end
end

function ezInterrupt:UNIT_SPELLCAST_START(event, unitID, spellName)
	if unitID == ezInterrupt.db.profile.tracking and UnitIsEnemy("player", unitID) and not select(9, UnitCastingInfo(unitID)) and self:IsSpellAllowed(spellName) and self:IsInterruptReady() then 
		self:StartAlert()
	end
end

function ezInterrupt:UNIT_SPELLCAST_CHANNEL_START(event, unitID, spellName)
	if unitID == ezInterrupt.db.profile.tracking and UnitIsEnemy("player", unitID) and not select(8, UnitChannelInfo(unitID)) and self:IsSpellAllowed(spellName) and self:IsInterruptReady() then 
		self:StartAlert()
	end
end

function ezInterrupt:PLAYER_TARGET_CHANGED()
	if UnitIsEnemy("player", "target") then
		local spellName = UnitCastingInfo("target") or UnitChannelInfo("target")
		local immune = select(9, UnitCastingInfo("target")) or select(8, UnitChannelInfo("target"))
		if spellName and not immune and self:IsSpellAllowed(spellName) and self:IsInterruptReady() then
			self:StartAlert()
		else
			self:StopAlert()
		end
	else
		self:StopAlert()
	end
end

--[[ disabled as it's not doing what one would expect
function ezInterrupt:UNIT_SPELLCAST_INTERRUPTIBLE(event, unitID, arg3, arg4)
	local msg = (event.." "..unitID.." "..GetUnitName(unitID).." "..tostring(arg3).." "..tostring(arg4))
	print(msg)
	message(msg)
	if unitID == ezInterrupt.db.profile.tracking and UnitIsEnemy("player", unitID) then
		local spellName = select(1, UnitCastingInfo("target")) or select(1, UnitChannelInfo("target"))
		if spellName ~= nil and self:IsSpellAllowed(spellName) and IsInterruptReady() then
			self:StartAlert()
		end
	end
end
]]--
--[[ disabled as it's not doing what one would expect
function ezInterrupt:UNIT_SPELLCAST_NOT_INTERRUPTIBLE(event, unitID, arg3, arg4)
	local msg = (event.." "..unitID.." "..GetUnitName(unitID).." "..tostring(arg3).." "..tostring(arg4))
	print(msg)
	message(msg)
	if unitID == ezInterrupt.db.profile.tracking then
		self:StopAlert()
	end
end
]]--

function ezInterrupt:UpdateFilterList(table, tableName, spellName) -- add/remove spells from/to blacklist/whitelist
	if spellName ~= nil and spellName ~= "" then
		if tContains(table, spellName) then
			for i,v in ipairs(table) do 
				if v == spellName then tremove(table, i) end
			end
			DEFAULT_CHAT_FRAME:AddMessage("|cFFff9000ez|r|cFFccff1dInterrupt:|r removed |cFF5dd9f8"..spellName.."|r from "..tableName)
		else 
			tinsert(table, spellName)
			DEFAULT_CHAT_FRAME:AddMessage("|cFFff9000ez|r|cFFccff1dInterrupt:|r added |cFF5dd9f8"..spellName.."|r to "..tableName)
		end	
	else
			DEFAULT_CHAT_FRAME:AddMessage("|cFFff9000ez|r|cFFccff1dInterrupt:|r invalid spell name")
	end
end

function ReplaceTags(str)
	return msgTags [str]
end


function ezInterrupt:PLAYER_ENTERING_WORLD(event)
	self:UpdateOutputChannel()
	self:UpdateCombatLogParsing()
end

function ezInterrupt:UpdateOutputChannel()
	if self.db.profile.slashRaid and GetNumRaidMembers() > 0 then 
		outputChannel = "RAID"
	elseif self.db.profile.slashParty and ((GetNumPartyMembers() > 0) or (GetNumRaidMembers() > 0)) then
		outputChannel = "PARTY"
	elseif self.db.profile.slashSay then 
		outputChannel = "SAY"
	else
		outputChannel = "NONE"
	end
end

function ezInterrupt:UpdateCombatLogParsing() -- enables/disables combat log parsing according to instance type and configuration settings
	local instanceType = select(2, IsInInstance())
	if (instanceType == "raid" and self.db.profile.useInRaid) or (instanceType == "party" and self.db.profile.useInParty) or (instanceType == "pvp" and self.db.profile.useInBattleground) or (instanceType == "arena" and self.db.profile.useInArena) or (instanceType == "none" and self.db.profile.useOutsideInstance) then
		if self.db.profile.enableAnnouncing then
			self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
		else
			self:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
		end
		if self.db.profile.enableWarning then
			frame:RegisterEvent("SPELL_UPDATE_COOLDOWN")
			self:RegisterEvent("UNIT_SPELLCAST_STOP")
			self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_STOP", "UNIT_SPELLCAST_STOP")
			self:RegisterEvent("UNIT_SPELLCAST_START")
			self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_START")
			self:RegisterEvent("PLAYER_TARGET_CHANGED")
			self:RegisterEvent("PLAYER_FOCUS_CHANGED", "PLAYER_TARGET_CHANGED")
			--self:RegisterEvent("UNIT_SPELLCAST_NOT_INTERRUPTIBLE")
			--self:RegisterEvent("UNIT_SPELLCAST_INTERRUPTIBLE")
		else
			frame:UnregistEvent("SPELL_UPDATE_COOLDOWN")
			self:UnregisterEvent("UNIT_SPELLCAST_START")
			self:UnregisterEvent("UNIT_SPELLCAST_STOP")
			self:UnregisterEvent("UNIT_SPELLCAST_CHANNEL_START")
			self:UnregisterEvent("UNIT_SPELLCAST_CHANNEL_STOP")
			self:UnregisterEvent("PLAYER_TARGET_CHANGED")
			self:UnregisterEvent("PLAYER_FOCUS_CHANGED")
			--self:UnregisterEvent("UNIT_SPELLCAST_NOT_INTERRUPTIBLE")
			--self:UnregisterEvent("UNIT_SPELLCAST_INTERRUPTIBLE")
		end
	else
		frame:UnregistEvent("SPELL_UPDATE_COOLDOWN")
		self:UnregisterEvent("SPELL_UPDATE_COOLDOWN")
		self:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
		self:UnregisterEvent("UNIT_SPELLCAST_START")
		self:UnregisterEvent("UNIT_SPELLCAST_STOP")
		self:UnregisterEvent("UNIT_SPELLCAST_CHANNEL_START")
		self:UnregisterEvent("UNIT_SPELLCAST_CHANNEL_STOP")
		self:UnregisterEvent("PLAYER_TARGET_CHANGED")
		self:UnregisterEvent("PLAYER_FOCUS_CHANGED")
		--self:UnregisterEvent("UNIT_SPELLCAST_NOT_INTERRUPTIBLE")
		--self:UnregisterEvent("UNIT_SPELLCAST_INTERRUPTIBLE")
	end
end

function ezInterrupt:COMBAT_LOG_EVENT_UNFILTERED(_ , _, eventType, sourceGUID, _, _, _, destName, _, spellID, _, _, extraID, ...)
	if eventType == "SPELL_INTERRUPT" and (sourceGUID == UnitGUID("player") or sourceGUID == UnitGUID("pet")) then
		msgTags ["[spell]"] 		= GetSpellLink(extraID)
		msgTags ["[target]"] 		= destName
		msgTags ["[interrupt]"] 	= GetSpellLink(spellID)
		local msg = string.gsub(self.db.profile.customMessage, "%[%a+%]", ReplaceTags)
		if self.db.profile.enableWhispering and self.db.profile.whisperTarget ~= nil then SendChatMessage(msg, "WHISPER", nil, self.db.profile.whisperTarget ) end
		if self.db.profile.enableCustomChannel and self.db.profile.customChannel ~= nil then SendChatMessage(msg, "CHANNEL", nil, self.db.profile.customChannel ) end	
		if outputChannel ~= "NONE" then SendChatMessage(msg, outputChannel) end
	end
end